indexing
	description: "Position on a TRIO Board"
	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	BOARD_POSITION

inherit
	ANY
		redefine
			copy, is_equal
		end

create
	make


feature {NONE} -- Initialization

	make (r : INTEGER; c : INTEGER) is
		-- Initialize `Current'.
		require
			coordinates_are_correct : is_in_range(r) and is_in_range(c)
		do
			row := r
			column := c
		end


feature -- Access
	min_value : INTEGER is 1 -- minimum allowed value in row or column

	row : INTEGER
	column : INTEGER


feature -- Status report

	is_in_range (coordinate : INTEGER) : BOOLEAN is
    	-- tells that `coordinate' is in correct range for a trio position
    	do
    		Result := (min_value <= coordinate)
	    end


feature -- Comparison

	is_equal (other : like Current) : BOOLEAN is
		do
			Result := (row = other.row) and (column = other.column)
		end


feature -- Conversion

	to_string : STRING is
  		-- find the string representation of `Current'
  		local
  			temp : STRING
  		do
  			create temp.make_from_string("[")
  			temp.append_integer(row)
  			temp.append(",")
  			temp.append_integer(column)
  			temp.append("]")
  			Result := temp
  		end


feature -- Duplication

	copy (other : like Current) is
		do
			row    := other.row
			column := other.column
		end


invariant
	isCorrect : is_in_range(row) and is_in_range(column)

end
