indexing
	description	: "Main window for this WEL Application"
	author		: "Generated by the New WEL Application Wizard"
	date		: "$Date: 2008/9/7 10:23:35 $"
	revision	: "1.0.0"

class
	MAIN_WINDOW

inherit
	WEL_FRAME_WINDOW
		redefine
			class_icon,
			class_background,
			on_control_id_command,
			on_menu_command,
			on_size,
			on_timer
		end

	RESOURCE_CONSTANTS
		export
			{NONE} all
		end

	WEL_COLOR_CONSTANTS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (trio_application : ROOT_CLASS) is
			-- Make the main window
		local
			bitmap_index : INTEGER
			button_size : INTEGER
			st : STRING
		do
			application      := trio_application
			resource_manager := application.resource_manager
			sound_manager    := application.sound_manager

			-- set main window properties
			st := resource_manager.get_cd_string(rsc_cd_string_mainwin_title)
			make_top(st)

			-- create a status window
			create status_window.make(Current, -1)
			status_window.set_multiple_mode

			-- create tool bar
			create tool_bar.make(Current, -1)
			button_size := rsc_mainwin_toolbar_size
			tool_bar.set_button_size(button_size, button_size)
			tool_bar.set_bitmap_size(button_size, button_size)
			create tool_bar_bitmap.make(rsc_id_mainwin_toolbar_bmp)
			tool_bar.add_bitmaps(tool_bar_bitmap, 1)
			bitmap_index := tool_bar.last_bitmap_index
			create start_button.make_button(bitmap_index + 0, rsc_id_cmd_start)
			create set_options_button.make_button(bitmap_index + 1, rsc_id_cmd_set_options)
			create high_score_button.make_button(bitmap_index + 2,
											     rsc_id_cmd_show_highscore)
			create instructions_button.make_button(bitmap_index + 3,
												   rsc_id_cmd_show_instructions)
			create stop_button.make_button(bitmap_index + 4, rsc_id_cmd_stop)

			tool_bar.add_buttons(<<set_options_button, high_score_button,
								   start_button, stop_button, instructions_button>>)
			tool_bar.auto_size

			-- create the board window
			create board_window.make(current)

			-- create the value text field
			create value_text_field.make(current, "", 0,0, 10,10, -1)
			value_text_field.set_read_only

			-- create the push button for no solutions
			create unsolvable_button.make(current, "", 0,0, 10,10,
			                              rsc_id_cmd_report_unsolvable)

			-- position_subwindows
			resize(450,560)
		end


feature -- Access

	game : GAME
	remaining_think_time : INTEGER  -- remaining think time in seconds
	displayed_message : STRING   -- current message displayed


feature -- Status change

	clear_timer_field is
			-- clear timer field
		local
			empty_string  : STRING
		do
			create empty_string.make_empty
			-- TODO: remove bug workaround
			-- status_window.set_text_part(status_window_timer_part, "")
			status_window.set_text_part(status_window_timer_part,
			                            empty_string.as_string_32)
		end

	-- --------------------

	reset_board_marking is
			-- resets all special marks on board_window
		local
			position_list : LIST[BOARD_POSITION]
		do
			create {LINKED_LIST[BOARD_POSITION]} position_list.make
			board_window.set_special_positions(position_list, 0, false)
		end

	-- --------------------

    set_unsolvable_button_visibility (is_visible : BOOLEAN) is
    		-- makes `unsolvable button' visible or invisible
    	local
    		new_height : INTEGER
    	do
    		if is_visible then
    			new_height := unsolvable_button_default_height
    		else
    			new_height := 0
    		end

   			unsolvable_button.set_height(new_height)
   			position_subwindows
    	end

	-- --------------------

	set_game (new_game : GAME) is
			-- show `new_game' as game
		local
			board : BOARD
		do
			-- clear score display and value field
			game := void
			update_score(0, 0, 0, 0)
			update_value_field

			-- set board
			game := new_game

			if game = Void then
				board := Void
			else
				board := game.board
			end

			board_window.set_board(board)
			board_window.invalidate

		end

	-- --------------------

	show_evaluation (position : BOARD_POSITION;  is_good : BOOLEAN) is
			-- shows evaluation of position on board as good or bad
		local
			position_list : LIST[BOARD_POSITION]
		do
			remaining_think_time := 0

			if position /= Void then
				create {LINKED_LIST[BOARD_POSITION]} position_list.make
				position_list.extend(position)
				board_window.set_special_positions(position_list, game.item, is_good)
			end
		end

	-- --------------------

	show_solutions is
			-- show all solutions for current board value
		require
			is_in_game : game /= void and then not game.after
			game_has_board : game.board /= void
		local
			board_width   : INTEGER
			i             : INTEGER
			j             : INTEGER
			has_solutions : BOOLEAN
			position      : BOARD_POSITION
			position_list : LIST[BOARD_POSITION]
		do
			position_list := game.board.get_solving_positions(game.item)
			board_width := game.board.width
			has_solutions := (position_list.count > 0)

			if not has_solutions then
				-- mark all positions as bad
				from
					i := 1
				until
					i = board_width + 1
				loop
					from
						j := 1
					until
						j = board_width + 1
					loop
						create position.make(i, j)
						position_list.extend(position)
						j := j + 1
					end
					i := i + 1
				end
			end

			board_window.set_special_positions(position_list, game.item,
			                                   has_solutions)
		end

	-- --------------------

	start_timer (think_time : INTEGER) is
			-- activate timer for the think time
		do
			remaining_think_time := think_time
			set_timer(one_second_timer, 1000)
		end

	-- --------------------

	stop_timer is
			-- deactivate think time timer
		do
			kill_timer(one_second_timer)
		end

	-- --------------------

	update_score (right_guess_count, wrong_guess_count, unsolvable_count : INTEGER;
                  score : INTEGER) is
			-- updates all score counts in status window and clears timer status window
		local
			f : FORMAT_INTEGER
			st : STRING
		do
			create f.make(2)
			st := resource_manager.get_cd_string(rsc_cd_string_mainwin_statusinfo)
			st := resource_manager.expand3(st,
			                               f.formatted(right_guess_count),
			                               f.formatted(wrong_guess_count),
			                               f.formatted(unsolvable_count))

			-- TODO: remove bug workaround
			-- status_window.set_text_part(status_window_counter_part, st)
			-- status_window.set_text_part(status_window_score_part, score.out)
			status_window.set_text_part(status_window_counter_part, st.as_string_32)
			status_window.set_text_part(status_window_score_part,
			                            score.out.as_string_32)
			st := resource_manager.get_cd_string(rsc_cd_string_mainwin_nosolutions)
			unsolvable_button.set_text(st)
		end

	-- --------------------

	update_value_field is
			-- show current game value
		local
			st : STRING
			value_text_log_font : WEL_LOG_FONT
			value_text_font : WEL_FONT
		do
			if game = Void or else game.after then
				st := ""
			else
				st := game.item.out
			end

			create value_text_log_font.make(value_text_field_height, "Arial")
			create value_text_font.make_indirect(value_text_log_font)
			value_text_field.set_font(value_text_font)
			value_text_field.set_text(st)
			value_text_field.invalidate
		end


feature -- Status report

	show_message (st : STRING) is
			-- show `st' as message
		do
			displayed_message := st
			-- TODO: remove bug workaround
			-- status_window.set_text_part(status_window_message_part, st)
			status_window.set_text_part(status_window_message_part, st.as_string_32)
		end


feature {NONE} -- Implementation

    application : ROOT_CLASS
    resource_manager : RESOURCE_MANAGER
	sound_manager : SOUND_MANAGER

	one_second_timer    : INTEGER is 1

    -- presentation elements
	tool_bar            : WEL_TOOL_BAR
	tool_bar_bitmap     : WEL_TOOL_BAR_BITMAP

	high_score_button   : WEL_TOOL_BAR_BUTTON
	instructions_button : WEL_TOOL_BAR_BUTTON
	set_options_button  : WEL_TOOL_BAR_BUTTON
	start_button        : WEL_TOOL_BAR_BUTTON
	stop_button         : WEL_TOOL_BAR_BUTTON

	unsolvable_button	: WEL_PUSH_BUTTON

	board_window        : BOARD_WINDOW
	status_window       : WEL_STATUS_WINDOW

	value_text_field    : WEL_SINGLE_LINE_EDIT

	-- some dimensions
	value_text_field_height  : INTEGER is 50
	value_text_field_width   : INTEGER is 60
	unsolvable_button_default_height : INTEGER is 30

	-- parts of status window
	status_window_timer_part   : INTEGER is 0
	status_window_counter_part : INTEGER is 1
	status_window_score_part   : INTEGER is 2
	status_window_message_part : INTEGER is 3


	class_background: WEL_BRUSH is
			-- White background
		once
			create Result.make_by_sys_color(Color_btnface + 1)
		end

	class_icon: WEL_ICON is
			-- Window's icon
		once
			create Result.make_by_id(rsc_id_application_ico)
		end

	-- --------------------

	position_subwindows is
			-- put subwindows in correct position
		local
			bw_x, bw_y, bw_w, bw_h : INTEGER
			first_row_bottom : INTEGER
			frame_width : INTEGER
			last_row_top : INTEGER
			right_edge_position_list : ARRAY[INTEGER]
			sw_x, sw_y, sw_h, sw_w : INTEGER
			ub_x, ub_y, ub_w, ub_h : INTEGER
			vt_x, vt_y, vt_w, vt_h : INTEGER
		do
			frame_width := 20

			-- put value text control near tool bar
			vt_w := value_text_field_width
			vt_h := value_text_field_height
			vt_x := 5 * rsc_mainwin_toolbar_size + 2 * frame_width
			vt_x := vt_x + (width - frame_width - vt_x - vt_w) // 2
			vt_y := 5
			value_text_field.move_and_resize(vt_x, vt_y, vt_w, vt_h, true)

			first_row_bottom := (tool_bar.height).max(vt_y + vt_h)

			-- put status window on bottom
			sw_x := frame_width
			sw_h := status_window.height
			sw_y := height - sw_h - frame_width
			sw_w := width - 2 * frame_width
			status_window.move_and_resize(sw_x, sw_y, sw_w, sw_h, true)
			create right_edge_position_list.make (1, 4)
			right_edge_position_list[1] := width // 12
			right_edge_position_list[2] := width // 3
			right_edge_position_list[3] := width // 2
			right_edge_position_list[4] := -1
			status_window.set_parts(right_edge_position_list)

			last_row_top := sw_y

			-- put board window
			bw_y := first_row_bottom + frame_width
			bw_w := sw_w
			bw_h := last_row_top - frame_width - bw_y

			ub_h := unsolvable_button.height

			if ub_h > 0 then
				bw_h := bw_h - ub_h - frame_width
			end

			bw_w := bw_w.min(bw_h)
			bw_h := bw_w
			bw_x := ((width - 2 * frame_width - bw_w) // 2).max(frame_width)
			board_window.move_and_resize(bw_x, bw_y, bw_w, bw_h, true)

			-- put unsolvable_button below the board
			ub_x := bw_x
			ub_y := last_row_top - unsolvable_button_default_height - frame_width
			ub_w := bw_w
			unsolvable_button.move_and_resize(ub_x,ub_y, ub_w,ub_h, true)
		end


feature -- Event Handling (public)

	notify_about_click (is_right_click : BOOLEAN;  position : BOARD_POSITION) is
			-- callback routine for board_window to tell about a
			-- mouse click at `position'; the type of click is
			-- specified by `is_right_click'
		do
			application.notify_about_click(is_right_click, position)
		end


feature {NONE} -- Event Handling (local)

	set_options is
			-- query for new think time
		do

		end

	-- --------------------

	on_control_id_command (control_id: INTEGER) is
		do
			on_menu_command (control_id)
		end

	-- --------------------

	on_menu_command (menu_id: INTEGER) is
			-- executes the command identified by `menu_id'.
		local
			user_wants_to_quit : BOOLEAN
		do
			inspect menu_id
			    when rsc_id_cmd_report_unsolvable then
					application.notify_about_click(false, void)

				when rsc_id_cmd_set_options then
					application.set_state(application.State_showing_preferences)

				when rsc_id_cmd_show_highscore then
					if remaining_think_time = 0 then
						-- show high score window only when user is not thinking
						application.show_high_score_window
					end

				when rsc_id_cmd_start then
					application.set_state(application.State_loading_game)

				when rsc_id_cmd_stop then
					user_wants_to_quit := application.show_quit_game_query_window

					if user_wants_to_quit then
						application.set_state(application.State_before_game)
					end

				when rsc_id_cmd_show_instructions then
					if remaining_think_time = 0 then
						-- show instructions window only when user is not thinking
						application.show_instructions_window
					end
				else
			end
		end

	-- --------------------

	on_size (size_type: INTEGER; a_width: INTEGER; a_height: INTEGER) is
    		-- resize to new width and height
    	do
    		position_subwindows
    	end

	-- --------------------

	on_timer (timer_id: INTEGER) is
			-- react on timer with `timer_id'.
		require else
			valid_timer: timer_id = one_second_timer
		local
			st : STRING
		do
			remaining_think_time := remaining_think_time - 1

			if remaining_think_time = 0 then
				st := ""
				application.set_state(application.state_timeout)
			else
				if remaining_think_time < 6 then
					sound_manager.play_sound(sound_manager.SoundKind_click)
				end

				-- restart timer for another second
				create st.make_empty
				st.append_integer(remaining_think_time)
				set_timer(one_second_timer, 1000)
			end

			-- TODO: remove bug workaround
			-- status_window.set_text_part(status_window_timer_part, st)
			status_window.set_text_part(status_window_timer_part, st.as_string_32)
		end

end -- class MAIN_WINDOW
